/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Mon Mr 10 2003
    copyright            : (C) 2003 by Ingmar Bauer
 ***************************************************************************/

using namespace std;

#include <iostream>
#include <vector>
#include <algorithm>

#include "matrix.h"
#include "datasource.h"
#include "matrixsource.h"

#include "pagerank_opt.h"
#include "clever.h"
#include "salsa.h"
#include "clever2.h"


int doPagerank(string);
int doClever(string);
int doSalsa(string);

int main(int argc, char *argv[]) {

  cout << "Testsystem fr linkbasiertes Ranking" << endl;
  cout << "(C) 2003 by Ingmar Bauer (ingmar@metacrawler.de)" << endl << endl;

  if(argc<3) {
    cout << "Run with: rank datasource algorithm" << endl;
    cout << "datasource = filename of data" << endl;
    cout << "algorithms = [pr|clever|salsa]" << endl;
    return(0);
  }

  string dsname = string(argv[1]);
  string alg = string(argv[2]);
  

  if(alg == "pr") {
    doPagerank(dsname);
  } else if(alg == "clever") {
    doClever(dsname);
    return(0);
  } else if(alg == "salsa") {
    doSalsa(dsname);
    return(0);
  } else {
    cout << alg << " is not a known algorithm, use: [pr|clever|salsa]" << endl;
    return(0);
  }


  return(1);
}

/****************************************************************************/
/*** Pagerank Berechnung                                                  ***/
/****************************************************************************/
int doPagerank(string dsname) {

  vector<float> res;
  datasource ds(dsname);

  if(&ds != NULL) {
    cout << "Datasource " << dsname << " opened." << endl;
  } else {
    cout << "Error opening datasource " << dsname << "." << endl;
    return(0);
  }

  int size = ds.getMatrixSize();
  cout << "Matrix size: " << size << endl;

  cout << "Computing Pagerank..." << endl;
  pagerank_opt pr2(ds, .85, .0001);
  res = pr2.calc();

  cout << "sorting results..." << endl;
  // prepare order vector
  vector<float> order(size);
  for(int i=0; i<size; i++) {
    order[i] = i;
  }

  // bubble sort
  int i,j=size;

  while(j > 0) {
    for(i=1; i<j; i++) {
      if(res[i-1] < res[i]) {
        swap(res[i], res[i-1]);
        swap(order[i], order[i-1]);
      }
    }
    j--;
  }

  // print results according to order vector
  for(i=0; i<size; i++) {
    cout << res[i] << ": " << ds.getURL((int)order[i]) << endl;
  }

  cout << "done." << endl;

  return(1);
}

/****************************************************************************/
/*** Clever/HITS Berechnung                                               ***/
/****************************************************************************/
int doClever(string dsname) {

  vector<float> res;

  datasource ds(dsname);

  if(&ds != NULL) {
    cout << "Datasource " << dsname << " opened." << endl;
  } else {
    cout << "Error opening datasource " << dsname << "." << endl;
    return(0);
  }

  int size = ds.getMatrixSize();
  cout << "Matrix size: " << size << endl;

  clever2 cl2(ds);
  cl2.createMatrices(dsname);
  cout << "Calculation matrices created..." << endl;

  cout << "Computing Clever/HITS Auhorities..." << endl;
  matrixsource ms_auth(dsname+".aut.mat");
  clever2 cl2calc_auth(ms_auth, 0.85, 0.001);
  res = cl2calc_auth.calc();

  cout << "sorting results..." << endl;
  // prepare order vector
  vector<float> order(size);
  for(int i=0; i<size; i++) {
    order[i] = i;
  }

  // bubble sort
  int i,j=size;

  while(j > 0) {
    for(i=1; i<j; i++) {
      if(res[i-1] < res[i]) {
        swap(res[i], res[i-1]);
        swap(order[i], order[i-1]);
      }
    }
    j--;
  }

  // print results according to order vector
  for(i=0; i<size; i++) {
    cout << res[i] << ": " << ds.getURL((int)order[i]) << endl;
  }


  cout << endl << endl << "Computing Clever/HITS Hubs..." << endl;
  matrixsource ms_hub(dsname+".hub.mat");
  clever2 cl2calc_hub(ms_hub, 0.85, 0.001);
  res = cl2calc_hub.calc();

  cout << "sorting results..." << endl;
  // prepare order vector
  for(int i=0; i<size; i++) {
    order[i] = i;
  }

  // bubble sort
  j=size;

  while(j > 0) {
    for(i=1; i<j; i++) {
      if(res[i-1] < res[i]) {
        swap(res[i], res[i-1]);
        swap(order[i], order[i-1]);
      }
    }
    j--;
  }

  // print results according to order vector
  for(i=0; i<size; i++) {
    cout << res[i] << ": " << ds.getURL((int)order[i]) << endl;
  }


  cout << "done." << endl;

  return(1);
}

/****************************************************************************/
/*** SALSA Berechnung                                                     ***/
/****************************************************************************/
int doSalsa(string dsname) {

  vector<float> res;

  datasource ds(dsname);

  if(&ds != NULL) {
    cout << "Datasource " << dsname << " opened." << endl;
  } else {
    cout << "Error opening datasource " << dsname << "." << endl;
    return(0);
  }

  int size = ds.getMatrixSize();
  cout << "Matrix size: " << size << endl;


  // salsa - create matrices
  salsa sl_create(ds);
  sl_create.createMatrices();
  sl_create.saveMatrices(dsname);
  cout << "Calculation matrices created..." << endl;

  // salsa - calculate authorities
  cout << "Computing SALSA Authorities..." << endl;
  matrixsource ms_auth(dsname+".aut");
  salsa sl_auth(ms_auth, 0.85, 0.001);
  res = sl_auth.calc_eigenvector();

  cout << "sorting results..." << endl;
  // prepare order vector
  vector<float> order(size);
  for(int i=0; i<size; i++) {
    order[i] = i;
  }

  // bubble sort
  int i,j=size;

  while(j > 0) {
    for(i=1; i<j; i++) {
      if(res[i-1] < res[i]) {
        swap(res[i], res[i-1]);
        swap(order[i], order[i-1]);
      }
    }
    j--;
  }

  // print results according to order vector
  for(i=0; i<size; i++) {
    cout << res[i] << ": " << ds.getURL((int)order[i]) << endl;
  }



  // salsa - calculate hubs
  cout << endl << endl << "Computing SALSA Hubs..." << endl;
  matrixsource ms_hub(dsname+".hub");
  salsa sl_hub(ms_hub, 0.85, 0.001);
  res = sl_hub.calc_eigenvector();

  cout << "sorting results..." << endl;
  // prepare order vector
  for(int i=0; i<size; i++) {
    order[i] = i;
  }

  // bubble sort
  j=size;

  while(j > 0) {
    for(i=1; i<j; i++) {
      if(res[i-1] < res[i]) {
        swap(res[i], res[i-1]);
        swap(order[i], order[i-1]);
      }
    }
    j--;
  }

  // print results according to order vector
  for(i=0; i<size; i++) {
    cout << res[i] << ": " << ds.getURL((int)order[i]) << endl;
  }


  cout << "done." << endl;

  return(1);
}




#ifdef NONE
    datasource ds("java_exp3");

    //int size = 3;

/*   
    matrix<float> adjMat(size);
    cout << "Matrix created." << endl;
    ds.getAdjascencyMatrix(adjMat);
*/


        

    cout << endl << endl;

/*    
    matrixsource ms2("test.aut");
    salsa sl2(ms2, 0.85, 0.001);
    vector<float> res = sl2.calc();
*/   
//    cout << "salsa finished" << endl;


    // PageRank
    
    cout << "Computing Pagerank..." << endl;
    pagerank_opt pr2(ds, .85, .0001);
    vector<float> res = pr2.calc();

    //PageRank pr(&adjMat, .85, .1);
    //vector<float> res = pr.calc();

/*
    clever clever(&adjMat, 5);
    clever.calc();
    vector<float> res = clever.getAuthorities();
*/

//    LinkCount lcount(&adjMat);
//    vector<float> res = lcount.calc();


    cout << "calculated." << endl;


    // Create new order-vector
    vector<float> order(size);
    for(int i=0; i<size; i++) {
	order[i] = i;
    }
    //generate(order.begin(), order.end(), inc(i));

    cout << "bubble sort...\n";
    // bubble sort
    int i, j=size;
    while(j > 0) {
	for(i=1; i<j; i++) {
	    if(res[i-1] < res[i]) {
		swap(res[i], res[i-1]);
		swap(order[i], order[i-1]);
	    }
	}
	j--;
    }

    cout << "order[0]" << order[0] << endl;

    // print results according to order vector
    for(i=0; i<size; i++) {
	cout << res[i] << ": " << ds.getURL((int)order[i]) << endl;
//	cout << res[i] << ": " << order[i] << endl;
    }


    //cout << "3adjMat.n: " << adjMat.n << endl;
    //cout << "3adjMat.m[5][0]: " << adjMat.m[5][0] << endl;


/*
    matrix<float> adjMat(3);
    adjMat.m[0][0] = 0.0;
    adjMat.m[1][0] = 0.5;
    adjMat.m[2][0] = 0.5;
    adjMat.m[0][1] = 1.0;
    adjMat.m[1][1] = 0.0;
    adjMat.m[2][1] = 0.0;
    adjMat.m[0][2] = 1.0;
    adjMat.m[1][2] = 0.0;
    adjMat.m[2][2] = 0.0;

    adjMat.print();

    PageRank pr(&adjMat);
    vector<float> res = pr.calc();

    vector<float>::iterator it;
    for(it=res.begin(); it!=res.end(); it++) {
	cout << *it << endl;
    }

    adjMat.print();
*/
    return(1);
}

#endif
