#!/usr/bin/perl -w

#
# Ingmar Bauer
#

use strict;

if(scalar(@ARGV) < 1) {
    print "viewidx.pl src_id\n";
    return 0;
}

my $src_id;
my $offs_refs;
my $n_refs;
my $offs_url;
my $offs_irefs;
my $n_irefs;
my $help;
my $stop = 0;

# Fileroutinen
open(f_index, "<test.idx") or die("index not open");
open(f_refs, "<test.ref") or die("refs not open");
open(f_url, "<test.url") or die("urls not open");
open(f_invrefs, "<test.inv.ref") or die("inv refs not open");
do {
    read(f_index, $help, length(pack("IISIIS", 0,0,0,0,0,0)));
    
    ($src_id, $offs_refs, $n_refs, $offs_url, $offs_irefs, $n_irefs) =
		unpack("IISIIS", $help);

    if($src_id == $ARGV[0]) {
	print "Found id $src_id:\n";
	print "\tOffset references: $offs_refs\n";
	print "\tNumber of references: $n_refs\n";
	print "\tOffset URL: $offs_url\n";
	print "\tOffset inverted references: $offs_irefs\n";
	print "\tNumber of inverted references: $n_irefs\n";
	print "\tURL: ".&getURL($offs_url)."\n\n";
	
	if($n_refs > 0) {
	    my @refs = &getRefs($offs_refs, $n_refs);
	    print "References: ";
	    my $i;
	    for($i=0; $i<scalar(@refs)-1; $i++) {
		print "$refs[$i], ";
	    }
	    print $refs[$i];
	}
	print "\n";
	


	if($n_irefs > 0) {
	    my @refs = &getInvRefs($offs_irefs, $n_irefs);
	    print "Inverted References: ";
	    my $i;
	    for($i=0; $i<scalar(@refs)-1; $i++) {
	        print "$refs[$i], ";
	    }
	    print $refs[$i];
	}
	
	print "\n\n";
	
	$stop=1;
    }
    
    	
} while(!eof(f_index) && !$stop);


sub getRefs() {
    my ($offs, $n) = @_;
    my $help;
    my @refs;
    my $i;
    
    seek(f_refs, $offs, 0);
    for($i=0; $i<$n; $i++) {
	read(f_refs, $help, 4);
	push(@refs, unpack("I", $help));
    }
    return @refs;
}

sub getURL() {
    my ($offs) = @_;
    
    seek(f_url, $offs, 0);
    return(readline(f_url));
}

sub getInvRefs() {
    my ($offs, $n) = @_;
    my $help;
    my @refs;
    my $i;
    
    seek(f_invrefs, $offs, 0);
    for($i=0; $i<$n; $i++) {
	read(f_invrefs, $help, 4);
	push(@refs, unpack("I", $help));
    }
    return @refs;
}
    
close(f_invrefs);
close(f_index);
close(f_refs);
close(f_url);
