/***************************************************************************
                          clever2.cpp  -  description
                             -------------------
    begin                : Die Apr 29 2003
    copyright            : (C) 2003 by Ingmar Bauer
    email                : ingmar@metacrawler.de
 ***************************************************************************/

#include <vector>
#include <string>

#include "matrix.h"
#include "datasource.h"
#include "matrixsource.h"
#include "clever2.h"

//#define DEBUG
#define CLEVER_DAMPENING

clever2::clever2(datasource &ds) {
  this->ds = &ds;
  this->size = this->ds->getMatrixSize();
}

clever2::clever2(matrixsource &ms, float c, float eps) {
  this->ms = &ms;
  this->c = c;
  this->eps = eps;
}
  
clever2::~clever2() {
}

/****************************************************************************/
/*** int createMatrices(string fname)                                     ***/
/***----------------------------------------------------------------------***/
/*** Erzeugen der Berechnungsmatrizen.                                    ***/
/****************************************************************************/
int clever2::createMatrices(string fname) {
  matrix<float> adj = matrix<float>(size);
  ds->getAdjascencyMatrix(adj);

  hub = matrix<float>(size);
  auth = matrix<float>(size);
  unsigned int x,y,i;
  
  // A*A^T
  // A^T*A
  
  // start operation
  for(y=0;y<size;y++) {
    for(x=0;x<size;x++) {
      // multiplication
      for(i=0; i<size; i++) {
          hub.m[y][x] += ( adj.m[y][i] * adj.m[x][i]);
          auth.m[y][x] += ( adj.m[i][x] * adj.m[i][y]);
      }
    }
  }

  hub.save(fname+string(".hub.mat"));
  auth.save(fname+string(".aut.mat"));
  return 1;
}

/****************************************************************************/
/*** int createMatrices(string fname)                                     ***/
/***----------------------------------------------------------------------***/
/*** Erzeugen der Berechnungsmatrizen.                                    ***/
/****************************************************************************/
vector<float> clever2::calc() {
  vector<float> src;
  vector<float> dest;
  int n_el;
  int i;
  matrix_element el;
  float s=eps+1;  // vector distance
  int line=0;

  // init vectors
  src.clear();
  for(i=0; i<ms->getSize(); i++) {
    src.insert(src.begin(), 1, i+1);
  }
  
  // normalize vector
  float sum=0.0;
  for(i=0;i<ms->getSize(); i++) {
    sum += (src[i]*src[i]);
  }
  for(i=0;i<ms->getSize(); i++) {
    src[i] /= sqrt(sum);
  }

  // start computation
  while(s > eps) {

    dest.clear();
    dest.insert(dest.begin(), ms->getSize(), 0.0);

    // let's start computing dominant eigenvector
    ms->reInitMatrix();

    int stop=0;

    do {
      n_el = ms->prepareNextRow(line);
      if(n_el == 0) {
        stop=1;
      }

      while(ms->getNextRowEl(el) > 0) {
        dest[line] += (src[el.col]*el.value);
      }
    } while(!stop);

    // dampening
    #ifdef CLEVER_DAMPENING
    #ifdef DEBUG
    cout << "dampening... (c=" << c << ")\n";
    #endif
    for(i=0;i<ms->getSize();i++) {
      dest[i] = c * dest[i] + ((1.0-c)/ms->getSize()); // dampening
    }
    #endif

    // normalize vector
    float sum=0.0;
    for(i=0;i<ms->getSize(); i++) {
      sum += (dest[i]*dest[i]);
    }
    for(i=0;i<ms->getSize(); i++) {
      dest[i] /= sqrt(sum);
    }

    // calculate vector distance
    #ifdef DEBUG
    cout << "calculating vector distance...\n";
    #endif

    s = 0.0;            // vector distance
    for(i=0; i<ms->getSize(); i++) {
      s+=fabs(dest[i]-src[i]);
    }

    cout << "Vectorabstand: " << s << endl;

    // loop
    src = dest;
  }

  return dest;
}
