/***************************************************************************
                          datasource.cpp  -  description
                             -------------------
    begin                : Mon Mr 10 2003
    copyright            : (C) 2003 by Ingmar Bauer
    email                : ingmar@metacrawler.de
 ***************************************************************************/

#include <iostream>
#include <fstream>
#include <iomanip>
#include <string>
#include <vector>

using namespace std;

//#define DEBUG

#include "matrix.h"

typedef vector<float> fltVect;
typedef vector<int> intVect;
typedef vector<vector <float> > fltMatrix;
typedef fltVect::iterator fltVectIt;

#include "datasource.h"

// feste Laenge eines Indexeintrages
#define INDEX_SIZE (4+4+2+4+4+2)


int datasource::getMatrixSize() {
  int pos_index = f_index->tellg();

  f_index->seekg(0, ios_base::end);
  int filesize = f_index->tellg();
  f_index->seekg(pos_index);

  return (filesize / INDEX_SIZE);
}

/*****************************************************************************/
/*** Constructor                                                           ***/
/*****************************************************************************/
datasource::datasource(string str_fname) {

#ifdef DEBUG
  cout << "DSrc_File::DSrc_File: running." << endl;
#endif

  ready = true;
  f_index = new std::ifstream( (str_fname+string(".idx")).c_str());
  if(!*f_index) {
    cerr << "Kann folgende Datei nicht oeffnen: " << str_fname << ".idx" << endl;
    ready=false;
  }

  f_refs = new std::ifstream( (str_fname+string(".ref")).c_str());
  if(!*f_refs) {
    cerr << "Kann folgende Datei nicht oeffnen: " << str_fname << ".ref" << endl;
    ready=false;
  }

  f_url = new std::ifstream( (str_fname+string(".url")).c_str());
  if(!*f_url) {
    cerr << "Kann folgende Datei nicht oeffnen: " << str_fname << ".url" << endl;
    ready=false;
  }

  f_irefs = new std::ifstream( (str_fname+string(".inv.ref")).c_str());
  if(!*f_irefs) {
    cerr << "Kann folgende Datei nicht oeffnen: " << str_fname << ".inv.ref" << endl;
    ready=false;
  }

  offs_ref = 0;
  offs_url = 0;
  offs_iref = 0;
  size = this->getMatrixSize();
}


/*****************************************************************************/
/*** Destructor                                                            ***/
/*****************************************************************************/
datasource::~datasource() {

#ifdef DEBUG
  cout << "DSrc_File::~DSrc_File: closing index files." << endl;
#endif
  ready=false;
  f_irefs->close();
  f_url->close();
  f_refs->close();
  f_index->close();
}


/*****************************************************************************/
/*** int DSrc_File::getRefs()                                              ***/
/***-----------------------------------------------------------------------***/
/*** Initialisiert fuer nextRef                                            ***/
/*****************************************************************************/
int datasource::getRefs() {
  f_index->seekg(0, ios_base::beg);
  f_refs->seekg(0, ios_base::beg);
  f_url->seekg(0, ios_base::beg);
  f_irefs->seekg(0, ios_base::beg);

  return(0);
}

/*****************************************************************************/
/*** int DSrc_File::nextRef(int &src_id, vector<int> &lnk_ids)             ***/
/***-----------------------------------------------------------------------***/
/*** Holt die jeweils naechste src_id mit den zugehoerigen ref_ids und     ***/
/*** fuellt die uebergebenen Variablen.                                    ***/
/*****************************************************************************/
int datasource::nextRef(unsigned int &src_id, vector<int> &lnk_ids) {

  if(ready) {
    unsigned int currpos = f_index->tellg();

#ifdef DEBUG
    cout << "currpos: " << currpos << endl;
#endif

    if(currpos >= size*INDEX_SIZE) {	// vectorsize * elements = filesize
      return -1;
    }

    // naechste Source Id lesen
    f_index->read((char *)&src_id, 4);

    // Offset fuer Refs lesen
    f_index->read((char *)&offs_ref, 4);

    // Anzahl der Refs lesen
    f_index->read((char *)&n_refs, 2);

    // Offset fuer URL lesen
    f_index->read((char *)&offs_url, 4);

    // ueberspringe die invertierten Refs
    f_index->seekg(+6, ios_base::cur);
    

#ifdef DEBUG
    cout.setf(ios_base::hex, ios_base::basefield);
    cout << "src_id  : 0x" << src_id << endl;
    cout << "offs_ref: 0x" << offs_ref << endl;
    cout << "offs_url: 0x" << offs_url << endl;
    cout << "n_refs  : 0x" << n_refs << endl;
#endif

    f_refs->seekg(offs_ref);
    lnk_ids.clear();
    int help=0;

    for(int i=0; i<n_refs; i++) {
      f_refs->read((char *)&help, 4);
      lnk_ids.push_back(help);
    }

    return lnk_ids.size();
  }
  return -1;
}


/*****************************************************************************/
/*** int nextRef(int &src_id, vector<int> &lnk_ids)                        ***/
/***-----------------------------------------------------------------------***/
/*** Holt die jeweils naechste src_id mit den zugehoerigen ref_ids und     ***/
/*** fuellt die uebergebenen Variablen.                                    ***/
/*****************************************************************************/
int datasource::nextRef(unsigned int &src_id, vector<int> &lnk_ids, vector<int> &backlnk_ids) {

  if(ready) {
    unsigned int currpos = f_index->tellg();

#ifdef DEBUG
    cout << "currpos: " << currpos << endl;
    //cout << "size   : " << size << endl;
#endif

    if(currpos >= size*INDEX_SIZE) {  // vectorsize * elements = filesize
      return -1;
    }

    // naechste Source Id lesen
    f_index->read((char *)&src_id, 4);

    // Offset fuer Refs lesen
    f_index->read((char *)&offs_ref, 4);

    // Anzahl der Refs lesen
    f_index->read((char *)&n_refs, 2);

    // Offset fuer URL lesen
    f_index->read((char *)&offs_url, 4);

    // Offset fuer invertierte Referenzen lesen
    f_index->read((char *)&offs_iref, 4);

    // Anzahl der invertieren Refs lesen
    f_index->read((char *)&n_irefs, 2);
    
#ifdef DEBUG
    cout.setf(ios_base::hex, ios_base::basefield);
    cout << "src_id  : 0x" << src_id << endl;
    cout << "offs_ref: 0x" << offs_ref << endl;
    cout << "offs_url: 0x" << offs_url << endl;
    cout << "n_refs  : 0x" << n_refs << endl;
#endif

    // Vorwaertsreferenzen lesen
    f_refs->seekg(offs_ref);
    lnk_ids.clear();
    int help=0;

    for(int i=0; i<n_refs; i++) {
      f_refs->read((char *)&help, 4);
      lnk_ids.push_back(help);
    }

    // Rueckwaertsreferenzen lesen
    f_irefs->seekg(offs_iref);
    backlnk_ids.clear();
    help=0;

    for(int i=0; i<n_irefs; i++) {
      f_irefs->read((char *)&help, 4);
      backlnk_ids.push_back(help);
    }

    return lnk_ids.size();
  }
  return -1;
}

/*****************************************************************************/
/*** string getURL(unsigned int id)                                        ***/
/***-----------------------------------------------------------------------***/
/*** Holt die URL mit der uebergebenen Id.                                 ***/
/*****************************************************************************/
string datasource::getURL(unsigned int id) {
  unsigned long pos_index = f_index->tellg();
  unsigned long pos_url = f_url->tellg();
  unsigned int src_id;
  string url;

  f_index->seekg(0, ios_base::beg); // file beginning

  while(!f_index->eof()) {
    f_index->read((char *)&src_id, 4);

    if(src_id == id) {
      f_index->seekg(+6, ios_base::cur);
      f_index->read((char *)&offs_url, 4);
      
      f_url->seekg(offs_url);
      char help[256];
      f_url->getline(help, 255, '\n');
      url = string(help);
      break;
    } else if(src_id > id) {  // stop it, we're too far already
      break;
    }

    f_index->seekg(+16, ios_base::cur);	// jump directly to next src_id
  }

  f_url->seekg(pos_url);
  f_index->seekg(pos_index);
  return url;
}

/*****************************************************************************/
/*** int getAdjMatrix(matrix<float> &m)                                    ***/
/*****************************************************************************/
int datasource::getAdjMatrix(matrix<float> &m) {

  unsigned int src_id;
  vector<int> lnk_ids;

  this->getRefs();		// init index reader

  while(this->nextRef(src_id, lnk_ids) != -1) {
    // now update the matrix
    vector<int>::iterator v;
    for(v=lnk_ids.begin();v!=lnk_ids.end(); v++) {
      float dummy = (float)(1.0/(float)lnk_ids.size());
      m.m[*v][src_id] = dummy;
    }
  }
  return(1);
}

/*****************************************************************************/
/*** int getAdjascencyMatrix(matrix<float> &m)                             ***/
/*****************************************************************************/
int datasource::getAdjascencyMatrix(matrix<float> &m) {
  unsigned int src_id;
  vector<int> lnk_ids;

  this->getRefs();    // init index reader

  while(this->nextRef(src_id, lnk_ids) != -1) {
    // now update the matrix
    vector<int>::iterator v;
    for(v=lnk_ids.begin();v!=lnk_ids.end(); v++) {
      float dummy = (float)(1.0); // /(float)lnk_ids.size());
      m.m[src_id][*v] = dummy;
    }
  }

  return(1);
}

/*****************************************************************************/
