/***************************************************************************
                          pagerank_opt.cpp  -  description
                             -------------------
    begin                : Mon Mr 10 2003
    copyright            : (C) 2003 by Ingmar Bauer
    email                : ingmar@metacrawler.de
 ***************************************************************************/

#include <iostream>
#include <vector>
#include <algorithm>
#include <functional>
#include <string>

#include "matrix.h"

using namespace std;

#include "datasource.h"
#include "pagerank_opt.h"

//#define DEBUG

pagerank_opt::pagerank_opt(datasource &ds, float c=0.85, float eps=0.01) {
  this->c = c;
  this->eps = eps;
  this->ds = &ds;
  this->size = this->ds->getMatrixSize();

  // initialize vectors
  src.clear();
  src.insert(src.begin(), size, 1.0/size);
}

pagerank_opt::~pagerank_opt() {
}

vector<float> pagerank_opt::calc(void) {
  unsigned int i, j;
  float s=100;
  unsigned int src_id;
  vector<int> lnk_ids;

  src.clear();
  src.insert(src.begin(), size, 1.0/size);

  while(s > eps) {
    dest.clear();
    dest.insert(dest.begin(), size, 0.0);

    ds->getRefs();

    while(ds->nextRef(src_id, lnk_ids) != -1) {
      for(j=0; j<lnk_ids.size(); j++) {
        dest[lnk_ids[j]]+=(src[src_id]/lnk_ids.size());
      }
    }

    #ifdef DEBUG
    cout << "dampening... (c=" << c << ")\n";
    #endif
    
    for(j=0;j<size;j++) {
      dest[j] = c * dest[j] + ((1.0-c)/size);	// dampening
    }

    #ifdef DEBUG
    cout << "calculating vector distance...\n";
    #endif
    
    s = 0.0;          // vector distance
    for(i=0; i<size; i++) {
      s+=fabs(dest[i]-src[i]);
    }
    cout << "Vectorabstand: " << s << endl;

    src = dest;
  }
  
  return dest;
}

void pagerank_opt::modifyZeroRows(void) {
  int state=0;

  for(unsigned int i=0; i<size; i++) {
    state=0;
    for(unsigned int j=0; j<size; j++) {
      if(a->m[j][i] > 0) {
        state=1;
        continue;
      }
    }
    if(state==0) {
      for(unsigned int k=0; k<size; k++) {
        a->m[k][i] = (float)(1.0/(float)size);
      }
    }
  }
}

void pagerank_opt::normalize(float limit=100.0) {
  vector<float>::iterator it;
  float max=0.0;
  
  for(unsigned int i=0; i<size; i++) {
    if(dest[i] > max)
      max = dest[i];
  }
  
  for(it=dest.begin(); it!=dest.end(); it++) {
    if(*it > 0) {
      *it = *it * limit / max;
    }
  }
}
