#include <iostream>
#include <vector>
#include <algorithm>
#include <functional>
#include <string>
//#include <sqlplus.hh>

#include "matrix.h"

using namespace std;

#include "datasource.h"
#include "pr2.h"


PageRank2::PageRank2(DataSource &ds, float c=0.85, float eps=0.01) {
    this->c = c;
    this->eps = eps;
    this->ds = &ds;
    this->size = this->ds->getMatrixSize();
    
    // initialize vectors
    src.clear();
    src.insert(src.begin(), size, 1.0/size);

}

PageRank2::~PageRank2() {
    cout << "destruction time" << endl;
}

vector<float> PageRank2::calc(void) {
    int i, j;
    float s=100;
    int src_id;
    vector<int> lnk_ids;

    src.clear();
    src.insert(src.begin(), size, 1.0/size);
    
    while(s > eps) {
        dest.clear();
	dest.insert(dest.begin(), size, 0.0);
	
	ds->getRefs();
	
	while(ds->nextRef(src_id, lnk_ids) != -1) {
	    cout << "pr2: src_id: " << src_id << endl;
	    cout << "pr2: lnk_ids.size(): " << lnk_ids.size() << endl;
	    //cout << "lnk_ids[0]: " << lnk_ids[0] << endl;
	
	    for(j=0; j<lnk_ids.size(); j++) {
	    
		cout << "dest[" << lnk_ids[j] << "] += " << src[src_id]/lnk_ids.size() << endl;
		
		dest[lnk_ids[j]]+=(src[src_id]/lnk_ids.size());
	    }
	    
	    //cout << "src[src_id]=" << src[src_id]/lnk_ids.size() << endl;
	    //cout << "dest[]=" << dest[lnk_ids[0]] << endl;
	}

	
	cout << "dampening... (c=" << c << ")\n";
	for(j=0;j<size;j++) {
	    //cout << "Before: dest[" << j << "]=" << dest[j] << endl;
	    dest[j] = c * dest[j] + ((1.0-c)/size);	// dampening
	    //cout << "Dampening: dest[" << j << "]=" << dest[j] << endl;
	}

	//for(i=0;i<size; i++) {
	//    cout << "dest[i]" << dest[i] << endl;	
	//}
	
	cout << "calculating vector distance...\n";
	s = 0.0;					// vector distance
	for(i=0; i<size; i++) {
	    s+=fabs(dest[i]-src[i]);
	}
	cout << "Vectorabstand: " << s << endl;

	src = dest;
    }
    
/*    vector<float>::iterator it = dest.begin();
    while(it != dest.end()) {
	cout << *it << " ";
	it++;
    }
    cout << endl;
*/

/*    
    modifyZeroRows();

    do {
	r1.clear();		// clear result vector
	r1.insert(r1.begin(), size, 0.0);

	for(i=0; i<size; i++) {
	    for(j=0; j<size; j++) {
		r1[i] += (float)(c * a->m[i][j]) * r0[j];
	    }
	    r1[i] += (float)(1.0-c)*(1.0/size);
	}
	
	// Stop?
	s = 0.0;
	for(i=0; i<size; i++) {
	    s+=fabs(r1[i]-r0[i]);
	}
	cout << "Vectorabstand: " << s << endl;
	
	r0 = r1;
    } while(s > eps);
*/    
    //sort(r1.begin(), r1.end(), greater<float>());
    //normalize();
    

    return dest;
}

void PageRank2::modifyZeroRows(void) {
    int state=0;
    
    for(int i=0; i<size; i++) {
        state=0;
        for(int j=0; j<size; j++) {
    	    if(a->m[j][i] > 0) {
	        state=1;
	        continue;
	    }
	}
	if(state==0) {
	    for(int k=0; k<size; k++) {
	        a->m[k][i] = (float)(1.0/(float)size);
	    }
	}
    }
}

void PageRank2::normalize(float limit=100.0) {
    vector<float>::iterator it;
    //float max=*(r1.begin());


    float max=0.0;
    for(int i=0; i<size; i++) {
	if(dest[i] > max)
	    max = dest[i];
    }

    
    //float *max = max_element(r1.begin(), r1.end(), greater<float>());
    //cout << "max: " << *max << endl;
    
    for(it=dest.begin(); it!=dest.end(); it++) {
	if(*it > 0) {
	    *it = *it * limit / max;
	}
    }
}
